Welcome to the Xanadu Quantum Codebook! This resource will help you learn quantum computing by using
Xanadu's quantum software framework PennyLane.

Please read both "panels" on this page before exploring the rest of the
Codebook. This panel contains traditional textbook content that supports the
coding exercises in the adjacent left panel.

## What is the Xanadu Quantum Codebook?

The Codebook is an experimental, exercise-based introduction to quantum
computing. Rather than just reading a textbook or tutorial, or executing some
pre-written code cells, the Codebook is an active learning resource in which you will
learn by doing.

## Is the Codebook for me?

This resource is intended for people who:
 - know how to program in Python and use `numpy`
 - know some basic linear algebra

You do not need to know any physics or quantum computing!

Below we list the specific concepts you should be familiar with.

### Python

  - basic control flow (`for`, `if`, `while`, etc.)
  - basic NumPy usage (creating arrays, adding/subtracting/multiplying
    arrays, manipulating them; the [NumPy documentation](https://numpy.org/learn/) is the best resource for this)
  - [decorators](https://pythonbasics.org/decorators/)
  - conceptual knowledge of functional programming is beneficial, but not a requirement

Familiarity with [PennyLane](https://pennylane.ai/) is not required, and will
be taught by this resource. 

### Linear algebra

 - vectors
 - matrices
 - complex numbers (in both rectangular and polar form)
 - eigenvalues/eigenvectors
 - linear combinations
 - inner products
 - matrix exponentials
 - Cartesian and polar coordinates

A few other mathematical concepts that will be useful are

 - basic trigonometry
 - basic probability theory (probability, conditional probability, and marginal probability)

[Khan Academy](https://www.khanacademy.org/) is an excellent resource for learning about the above.
[Interactive Linear Algebra](http://textbooks.math.gatech.edu/ila/) is also another great option.

## How do I use the Codebook?

The content is divided up into a graph of **modules**, each containing a graph
of multiple **nodes**, like this one. (Note: modules are sometimes referred to
as *chapters*, and nodes as *sections*). The main screen provides three panels:
a graph view, coding challenges (codercises), and the textbook content.

The nodes in the graph will be different colours:

 - Available, incomplete nodes will display as white circles
 - The currently-selected node will be blue.
 - Nodes will turn green upon completion of all coding challenges within

To enter a node, simply click on it. 
You can go "back" to a previous graph using the arrow at the top, and entirely
collapse the graph pane using the arrow at the bottom.

Within a node, the middle pane (to the right of the graph) contains autograded
codercises. Initially, only a single codercise is visible, and subsequent ones
are unlocked upon successful completion. See the middle pane of this tutorial node
for more details about codercises.

On the right panel of each node you'll find companion "textbook" pages that will
help you with solving the coding exercises. They also contain many pen-and-paper
exercises which you can do at your leisure. The textbook component is meant to
be a standalone resource, so you can read through the material at your own pace,
and drop into whichever nodes and topics you are most interested in. However,
within a module, the content progresses in a relatively linear manner, so we
do recommend working through the nodes in order.

You can close the textbook at any time to focus on the coding challenge by clicking the
"X" in the top right corner. You can then click the "Show Textbook" prompt to
bring the pane back into view (similarly, you can close the codercises and
focus just on the textbook content).

---

***Exercise T.1*** These exercises are all optional, but we encourage you
nevertheless to work through them. All pen-and-paper exercises come with
solutions. Click the arrow below to view the "solution".

<details>
  <summary><i>Solution.</i></summary>

For some problems, hints will also be provided before the solution, using
this same dropdown format.

</details>

---

## Codebook contents

There are currently 5 modules in the Codebook, each indexed by a letter:

 - **T**, this **T**utorial section
 - **I**, **I**ntroduction to quantum computing and PennyLane
 - **A**, introduction to quantum **A**lgorithms
 - **G**, **G**rover's algorithm
 - **H**, **H**amiltonian simulation

You can navigate through the content in any order, and start working on
codercises in any node whenever you like. If you are a beginner, we recommend
starting in I, then moving downwards through the graph by progressing through
either A and G, or through H, depending on your interests.

New content will be added over time, consisting of both updates to existing
nodes and modules, as well as entirely new modules.
